import 'package:analyzer/dart/element/element.dart';
import 'package:source_gen/source_gen.dart';

abstract class BaseGeneratorForAnnotation<A, K> extends GeneratorForAnnotation<A> {

  final annotationChecker = TypeChecker.fromRuntime(A);
  final keyAnnotationChecker = TypeChecker.fromRuntime(K);

  /// Check if the classElement has a default constructor like MyClass()
  checkDefaultConstructor(ClassElement classElement, {String annotationName = "this annotation"}) {
    bool hasDefaultConstructor = false;
    classElement.constructors?.forEach((constructor) {
      if (constructor?.isDefaultConstructor ?? false) {
        hasDefaultConstructor = true;
        return;
      }
    });
    if (!hasDefaultConstructor)
      throw InvalidGenerationSourceError(
          'To use $annotationName make sure '
          'to have a default constructor in your class '
          '${classElement.displayName}.',
          todo: 'Add a datult constructor like MyClass() or '
              'MyClass({String param1, String param2}) to your '
              '${classElement.displayName} or remove the $annotationName '
              'annotation from it.',
          element: classElement);
  }

  /// Get ClassElement from generic Element if possible, otherwise throws
  /// exception
  ClassElement getClassElement(Element element,
      {String annotationName = "this annotation"}) {
    if (element is! ClassElement)
      throw InvalidGenerationSourceError(
          'Generator cannot target '
          '${element.displayName}. $annotationName can only be applied to a '
          'class.',
          todo: 'Check the type of ${element.displayName} or remove the '
              'annotation from it.',
          element: element);
    return element as ClassElement;
  }
}
